#include <stdio.h>
#include "mpi.h"

#define MIDPLANE 512
#define DEBUG

#if defined (_XPMPI)
#define MPI_Cart_create XPMPI_Cart_create
#endif /* (_XPMPI) */

#if !defined (_MPI)
#define MPI_Comm_rank PMPI_Comm_rank
#define MPI_Comm_size PMPI_Comm_size
#define MPI_Comm_group PMPI_Comm_group
#define MPI_Group_incl PMPI_Group_incl
#define MPI_Comm_create PMPI_Comm_create
#endif /* (_MPI) */

inline int get_rank(int x, int y, int z, int t, int * dims)
{
	int rank;

	rank = x;
	rank += (dims[0] * y);
	rank += (dims[0] * dims[1] * z);
	rank += (dims[0] * dims[1] * dims[2] * t);

	return rank;
}

void create_bgdims(int * bgdims, int size)
{
	char orig[1000], name[1000];
	int namelen, i, j, index;

	MPI_Get_processor_name(orig, &namelen);
	i = 0;
	while (orig[i] != '<') i++; i++;
	while (orig[i] != '<') i++; i++;

	for (index = 0; index < 4; index++) {
		j = 0;
		while ((orig[i] != ',') && (orig[i] != '>')) {
			if (orig[i] == ' ') {
				i++;
				continue;
			}
			name[j++] = orig[i++];
		}
		name[j] = '\0'; i++;
		bgdims[index] = atoi(name);
	}
}


int MPI_Cart_create(MPI_Comm comm_old, int ndims, int * dims, int * periods,
		    int reorder, MPI_Comm * comm_cart)
{
	int index, t, bgdims[4];
	int i, j, k, p;
	int cur_x, cur_y, cur_z;
	int start_x, start_y, start_z;

	MPI_Group comm_group, new_group;
	MPI_Comm comm_new;
	int * reordered_ranks, comm_size;

	int rank;

	if ((ndims != 2) || (periods[0] == 1) || (periods[1] == 1)) {
		fprintf(stderr, "MPI_Cart_create parameters too complicated to handle; passing on to MPICH2\n");
		return PMPI_Cart_create(comm_old, ndims, dims, periods, reorder, comm_cart);
	}

#ifdef DEBUG
	MPI_Comm_rank(comm_old, &rank);
#endif /* DEBUG */
	MPI_Comm_size(comm_old, &comm_size);
	if (!(reordered_ranks = (void *) malloc(comm_size * sizeof(int)))) {
		fprintf(stderr, "Malloc failure\n");
		MPI_Abort(MPI_COMM_WORLD, -1);
	}

	create_bgdims(bgdims, comm_size);
#ifdef DEBUG
	if (!rank) printf("Bgdims: %d %d %d %d\n", bgdims[0], bgdims[1], bgdims[2], bgdims[3]);
#endif /* DEBUG */
	t = bgdims[3];

	index = 0;
	cur_x = 1; cur_y = 1; cur_z = 1;
	start_x = 0; start_y = 0; start_z = 0;
	while (1) {
		for (k = start_z; k < cur_z; k++)
			for (j = start_y; j < cur_y; j++)
				for (i = start_x; i < cur_x; i++)
					for (p = 0; p < t; p++)
						reordered_ranks[index++] = get_rank(i, j, k, p, bgdims);
		if (index == comm_size) break;

		if (cur_x * cur_y * cur_z >= MIDPLANE) {
			start_z = cur_z;
			start_y = 0;
			start_x = 0;
			cur_z *= 2;
		}
		else if ((cur_x == cur_z) && (cur_x < bgdims[0])) {
			start_x = cur_x;
			start_y = 0;
			start_z = 0;
			cur_x *= 2;
		}
		else if ((cur_y == cur_z) && (cur_y < bgdims[1])) {
			start_y = cur_y;
			start_x = 0;
			start_z = 0;
			cur_y *= 2;
		}
		else if (cur_z < bgdims[2]) {
			start_z = cur_z;
			start_y = 0;
			start_x = 0;
			cur_z *= 2;
		}
		else {
			printf("Bad dimension values: (%d,%d,%d -- %d)\n", bgdims[0], bgdims[1], bgdims[2], index);
			MPI_Abort(MPI_COMM_WORLD, -1);
		}
	}

	MPI_Comm_group(MPI_COMM_WORLD, &comm_group);
	MPI_Group_incl(comm_group, comm_size, reordered_ranks, &new_group);
	MPI_Comm_create(MPI_COMM_WORLD, new_group, &comm_new);

	return PMPI_Cart_create(comm_new, ndims, dims, periods, reorder, comm_cart);
}
