! This file is part of P3DFFT library
!
! Version 1.0 beta
!
! Copyright (C) 2006 Dmitry Pekurovsky, Giridhar Chukkapalli
!
!    P3DFFT is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.

!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

!----------------------------------------------------------------------------

! global variables
      module p3dfft

      include 'mpif.h'

      private

#ifdef DOUBLE_PREC
       integer, parameter,public :: mytype=8
       integer, parameter,public:: mpireal = MPI_DOUBLE_PRECISION
       integer,parameter,public:: mpicomplex = MPI_DOUBLE_COMPLEX
#else
       integer, parameter,public :: mytype=4
       integer, parameter,public:: mpireal = MPI_REAL
       integer,parameter,public:: mpicomplex = MPI_COMPLEX
#endif

      integer, public :: padd

      integer,save :: NX_fft,NY_fft,NZ_fft,numtasks,iproc,jproc
      integer,save :: ipid,jpid,taskid
      integer,save :: iistart,iiend,iisize,jjstart,jjsize,jjend
      integer,save ::jistart,kjstart,jisize,kjsize,jiend,kjend

      integer,save ::  nxh,nxhp

! mpi process info
!
      logical :: mpi_set=.false.
      integer, save :: mpi_comm_cart      
      integer, save :: mpi_comm_row, mpi_comm_col
      integer,save, dimension(:), allocatable :: iist,iien,iisz
      integer,save, dimension(:), allocatable :: jist,jien,jisz
      integer,save, dimension(:), allocatable :: jjst,jjen,jjsz
      integer,save, dimension(:), allocatable :: kjst,kjen,kjsz

! mpi derived data types for implementing alltoallv using send-recvs
      integer,save,dimension(:),allocatable:: IfSndCnts,IfSndStrt
      integer,save,dimension(:),allocatable:: IfRcvCnts,IfRcvStrt
      integer,save,dimension(:),allocatable:: KfSndCnts,KfSndStrt
      integer,save,dimension(:),allocatable:: KfRcvCnts,KfRcvStrt
      integer,save,dimension(:),allocatable:: JrSndCnts,JrSndStrt
      integer,save,dimension(:),allocatable:: JrRcvCnts,JrRcvStrt
      integer,save,dimension(:),allocatable:: KrSndCnts,KrSndStrt
      integer,save,dimension(:),allocatable:: KrRcvCnts,KrRcvStrt
      integer,save,dimension(:,:),allocatable:: status

      public :: get_dims,p3dfft_setup,ftran_r2c,btran_c2r

      contains

      subroutine get_dims(istart,iend,isize,conf)

      integer istart(3),iend(3),isize(3),conf
      
      if(.not. mpi_set) then
         print *,'P3DFFT error: call setup before other routines'
      else

      if(conf .eq. 1) then
         istart(1) = 1
         iend(1) = NX_fft
         isize(1) = NX_fft
         istart(2) = jistart
         iend(2) = jiend
         isize(2) = jisize
         istart(3) = kjstart
         iend(3) = kjend
         isize(3) = kjsize
      else if(conf .eq. 2) then
         istart(1) = iistart
         iend(1) = iiend
         isize(1) = iisize
         istart(2) = jjstart
         iend(2) = jjend
         isize(2) = jjsize
         istart(3) = 1
         iend(3) = NZ_fft
         isize(3) = NZ_fft
      endif

      endif
      end subroutine get_dims

c =========================================================
      subroutine p3dfft_setup(dims,nx,ny,nz)
c========================================================

      implicit none

      integer i,j,k,nx,ny,nz
      integer ierr, dims(2),  cartid(2)
      logical periodic(2),remain_dims(2)
      integer impid, ippid, jmpid, jppid

      mpi_set = .true.
      nx_fft = nx
      ny_fft = ny
      nz_fft = nz
      nxh=nx/2
      nxhp=nxh+1

      call MPI_COMM_SIZE (MPI_COMM_WORLD,numtasks,ierr)
      call MPI_COMM_RANK (MPI_COMM_WORLD,taskid,ierr)

      iproc = dims(1)
      jproc = dims(2)
      periodic(1) = .false.
      periodic(2) = .false.
! creating cartesian processor grid
      call MPI_Cart_create(MPI_COMM_WORLD,2,dims,periodic,
     &     .false.,mpi_comm_cart,ierr)
! Obtaining process ids with in the cartesian grid
      call MPI_Cart_coords(mpi_comm_cart,taskid,2,cartid,ierr)
! process with a linear id of 5 may have cartid of (3,1)
      ipid = cartid(1)
      jpid = cartid(2)
! here i is east-west j is north-south
! impid is west neighbour ippid is east neighbour and so on
      impid = ipid - 1
      ippid = ipid + 1
      jmpid = jpid - 1
      jppid = jpid + 1
!boundary processes
      if (ipid.eq.0) impid = MPI_PROC_NULL
      if (jpid.eq.0) jmpid = MPI_PROC_NULL
      if (ipid.eq.iproc-1) ippid = MPI_PROC_NULL
      if (jpid.eq.jproc-1) jppid = MPI_PROC_NULL
! using cart comworld create east-west(row) sub comworld
      remain_dims(1) = .true.
      remain_dims(2) = .false.
      call MPI_Cart_sub(mpi_comm_cart,remain_dims,mpi_comm_row,ierr)
! using cart comworld create north-south(column) sub comworld
      remain_dims(1) = .false.
      remain_dims(2) = .true.
      call MPI_Cart_sub(mpi_comm_cart,remain_dims,mpi_comm_col,ierr)

      allocate (iist(0:iproc-1))
      allocate (iisz(0:iproc-1))
      allocate (iien(0:iproc-1))
      allocate (jjst(0:jproc-1))
      allocate (jjsz(0:jproc-1))
      allocate (jjen(0:jproc-1))
      allocate (jist(0:iproc-1))
      allocate (jisz(0:iproc-1))
      allocate (jien(0:iproc-1))
      allocate (kjst(0:jproc-1))
      allocate (kjsz(0:jproc-1))
      allocate (kjen(0:jproc-1))
!
!Mapping 3-D data arrays onto 2-D process grid
! (nx+2,ny+2,nz) => (iproc,jproc)      
! 
      call MapDataToProc(nxh,iproc,iist,iien,iisz)
      call MapDataToProc(ny,iproc,jist,jien,jisz)
      call MapDataToProc(ny,jproc,jjst,jjen,jjsz)
      call MapDataToProc(nz,jproc,kjst,kjen,kjsz)


      iistart = iist(ipid)
      jjstart = jjst(jpid)
      jistart = jist(ipid)
      kjstart = kjst(jpid)
      iisize= iisz(ipid)
      jjsize= jjsz(jpid)
      jisize= jisz(ipid)
      kjsize= kjsz(jpid)
      iiend = iien(ipid)
      jjend = jjen(jpid)
      jiend = jien(ipid)
      kjend = kjen(jpid)

      padd = max(iisize*jjsize*nz_fft,iisize*ny_fft*kjsize) - nxh*jisize*kjsize
      if(padd .lt. 0) then 
         padd=0
      else
         padd = padd / (nxhp*jisize)
      endif

!      print *,iisize,jjsize,jisize,kjsize

      
! Displacements and buffer counts for mpi_alltoallv

      allocate (IfSndStrt(0:iproc-1))
      allocate (IfSndCnts(0:iproc-1))     
      allocate (IfRcvStrt(0:iproc-1))
      allocate (IfRcvCnts(0:iproc-1))

      allocate (KfSndStrt(0:jproc-1))
      allocate (KfSndCnts(0:jproc-1))     
      allocate (KfRcvStrt(0:jproc-1))
      allocate (KfRcvCnts(0:jproc-1))

      allocate (JrSndStrt(0:jproc-1))
      allocate (JrSndCnts(0:jproc-1))     
      allocate (JrRcvStrt(0:jproc-1))
      allocate (JrRcvCnts(0:jproc-1))

      allocate (KrSndStrt(0:iproc-1))
      allocate (KrSndCnts(0:iproc-1))     
      allocate (KrRcvStrt(0:iproc-1))
      allocate (KrRcvCnts(0:iproc-1))

!   start pointers and types of send  for the 1st forward transpose
      do i=0,iproc-1
         IfSndStrt(i) = (iist(i) -1)* jisize*mytype*2
         IfSndCnts(i) = iisz(i) * jisize*mytype*2

!   start pointers and types of recv for the 1st forward transpose
         IfRcvStrt(i) = (jist(i) -1) * iisize*mytype*2
         IfRcvCnts(i) = jisz(i) * iisize*mytype*2
      end do

!   start pointers and types of send  for the 2nd forward transpose
      do i=0,jproc-1
         KfSndStrt(i) = (jjst(i) -1)*iisize*kjsize*mytype*2
         KfSndCnts(i) = iisize*kjsize*jjsz(i)*mytype*2

!   start pointers and types of recv for the 2nd forward transpose
         KfRcvStrt(i) = (kjst(i) -1) * iisize * jjsize*mytype*2
         KfRcvCnts(i) = iisize*jjsize*kjsz(i)*mytype*2
      end do

!   start pointers and types of send  for the 1st inverse transpose
      do i=0,jproc-1
         JrSndStrt(i) = (kjst(i) -1) * iisize * jjsize*mytype*2
         JrSndCnts(i) = iisize*jjsize*kjsz(i)*mytype*2

!   start pointers and types of recv for the 1st inverse transpose
         JrRcvStrt(i) = (jjst(i) -1)*iisize*kjsize*mytype*2
         JrRcvCnts(i) = jjsz(i) * iisize * kjsize*mytype*2
      end do

!   start pointers and types of send  for the 2nd inverse transpose
      do i=0,iproc-1
         KrSndStrt(i) = (jist(i) -1) * iisize*mytype*2
         KrSndCnts(i) = jisz(i) * iisize*mytype*2

!   start pointers and types of recv for the 2nd inverse transpose
         KrRcvStrt(i) = (iist(i) -1) * jisize*mytype*2
         KrRcvCnts(i) = jisize*iisz(i)*mytype*2
      enddo

      end subroutine

!==================================================================       
      subroutine MapDataToProc (data,proc,st,en,sz)
c========================================================
!    
       implicit none
       integer data,proc,st(0:proc-1),en(0:proc-1),sz(0:proc-1)
       integer i,size,nl,nu

       size=data/proc
       nu = data - size * proc
       nl = proc - nu
       st(0) = 1
       sz(0) = size
       en(0) = size
       do i=1,nl-1
         st(i) = st(i-1) + size
         sz(i) = size
         en(i) = en(i-1) + size
      enddo
      size = size + 1
      do i=nl,proc-1
         st(i) = en(i-1) + 1
         sz(i) = size
         en(i) = en(i-1) + size
      enddo
      en(proc-1)= data 
      sz(proc-1)= data-st(proc-1)+1

      end subroutine

c========================================================
c  3D FFT inverse transform with 2D domain decomposition
c
c  This version uses MPI_Alltoallv to exchange data between processors
c  In the second step, y planes are sent separately
c  The order of array elements in memory is the same in all stages: (x,y,z) 

c Input: XYZg - comlpex array, with y dimension contained entirely,
c               while x and y are block-distributed among processors in 2D grid
c Output: XgYZ - an array of real, x dimension is contained entirely within 
c processors memory  while z and y are block-distributed among processors 
c in 2D grid
c
c !!! CAUTION: In this version: all arrays may occupy the same memory space
c In this case their first elements should coincide. 
c Naturally, output overwrites input
c

      subroutine btran_c2r (XYZg,XgYZ,flg_inplace)
c========================================================

      use fft_spec
      implicit none

      real(mytype) XgYZ(nx_fft,jistart:jiend,kjstart:kjend)
c      complex(mytype) XZgY(iistart:iiend,ny,kjstart:kjend)
      complex(mytype) XYZg(iistart:iiend,jjstart:jjend,nz_fft)
      complex(mytype), allocatable :: XYgZ(:,:,:)

      integer x,y,z,i,err,nx,ny,nz
      logical flg_inplace

      if(.not. mpi_set) then
         print *,'P3DFFT error: call setup before other routines'
         return
      endif
      nx = nx_fft
      ny = ny_fft
      nz = nz_fft

      call init_work(nx,ny,nz)

      allocate(XYgZ(nxhp,jistart:jiend,kjstart:kjend+padd),stat=err)
      if(err .ne. 0) then
         print *,'btran_c2r(): Error ',err,' allocating array XYgZ'
      endif

      if(jproc .gt. 1) then

c         allocate(XYgZ(iistart:iiend,ny,kjstart:kjend))
      
            call init_b_c(XYZg, iisize*jjsize, 1,
     &        XYZg, iisize*jjsize, 1,nz,iisize*jjsize,.true.)
c Transform in z dimension for all x and y
c

            call exec_b_c(XYZg, iisize*jjsize, 1,
     &        XYZg, iisize*jjsize, 1,nz,iisize*jjsize)

c Exhange z-y pencils for all x
            call bcomm1(XYZg,XYZg)

      else
c In case of 1D decomposition do only FFT in Z without data exchange

            call init_b_c(XYZg,iisize*ny,1,XYZg,
     &       iisize*ny, 1, nz,iisize*ny,.true.)
            call exec_b_c(XYZg,iisize*ny,1,XYZg,
     &       iisize*ny,1, nz,iisize*ny)
      endif

      call clean_x2

      if(iproc .gt. 1) then
c
c Transform in y dimension for all x, one z-plane at a time
c
         call init_b_c(XYZg,iisize,1,XYgZ,iisize,1,ny,iisize,.true.)
  
         do z=kjstart,kjend

c            call exec_b_c(XYZg,iisize,1,XYgZ,iisize,1,ny,iisize)

               call btran_y_zplane(XYZg,z-kjstart,iisize,kjsize,iisize,1,
     &            XYgZ,z-kjstart,iisize,kjsize,iisize,1,ny,iisize) 
            
c             
c              call sexec_b_c(XZgY(iistart,1,z),iisize,1,
c     &              XZgY(iistart,1,z),iisize,1,ny,iisize)

         enddo

c         deallocate(XYgZ)   
c          allocate(buf(nxhp,jistart:jiend,kjstart:kjend))

         call bcomm2(XYgZ,XYgZ)

      else

            call init_b_c(XYZg,nxh,1,XYgZ,nxhp,1,ny,nxh,.true.)

c In case of 1D decomposition do only FFT in Y without data exchange
            do z=kjstart,kjend

c            call exec_b_c(XYgZ,nxh,1,XYgZ,nxhp,1,ny,nxh)

               call btran_y_zplane(XYZg,z-kjstart,nxh,kjsize,nxh,1,XYgZ,
     &      z-kjstart,nxhp,kjsize, nxhp,1,ny,nxh)

c               call sexec_b_c(XgYZ(1,1,z),nxh,1,buf(1,1,z),nxhp,1,ny,nxh)

               do y=1,ny
                  XYgZ(nxhp,y,z)=cmplx(imag(XYgZ(1,y,z)),0.0)
                  XYgZ(1,y,z) = cmplx(real(XYgZ(1,y,z)),0.0)
               enddo
            enddo

      endif

      call clean_x2

c Perform Complex-to-real FFT in x dimension for all y and z

      call init_b_c2r(XYgZ,nxhp,XgYZ,nx,nx,jisize*kjsize,.false.)
      call exec_b_c2r(XYgZ,nxhp,XgYZ,nx,nx,jisize*kjsize)

      call clean_x1

      deallocate(XYgZ)

      call free_work

      return
      end subroutine



c========================================================
c 3D FFT Forward transform, 2D domain decomposition
c
c  This version uses MPI_Alltoallv to exchange data between processors
c  In the first step, y planes are sent separately
c  The order of array elements in memory is the same in all stages: (x,y,z) 
c
c Input: XgYZ - an array of real, x dimension is contained entirely within 
c  processors memory  while z and y are block-distributed among processors 
c  in 2D grid XgYZ 
c Output: XYZg - comlpex array, with y dimension contained entirely,
c               while x and y are block-distributed among processors in 2D grid
c
c !!! CAUTION: In this version: all arrays may occupy the same memory space
c In this case their first elements should coincide. 
c Naturally, output overwrites input

      subroutine ftran_r2c (XgYZ,XYZg,flg_inplace)
c========================================================

      use fft_spec
      implicit none

c      real(mytype), pointer:: XgYZ(:,:,:)
c      complex(mytype),pointer :: XYZg(:,:,:)
      real(mytype) XgYZ(nx_fft,jistart:jiend,kjstart:kjend)
      complex(mytype) XYZg(iistart:iiend,jjstart:jjend,nz_fft)
      complex(mytype), allocatable :: XYgZ(:,:,:)
      integer x,y,z,i,err,nx,ny,nz
      logical flg_inplace


      if(.not. mpi_set) then
         print *,'P3DFFT error: call setup before other routines'
         return
      endif

      nx = nx_fft
      ny = ny_fft
      nz = nz_fft
c
c FFT transform in x for all z and y
c

      allocate(XYgZ(nxhp,jisize,kjsize+padd),stat=err)
      if(err .ne. 0) then
         print *,'ftran_r2c(): Error ',err,' allocating array XYgZ'
      endif

      call init_work(nx,ny,nz)

      call init_f_r2c(XgYZ,nx,XYgZ,nxhp,nx,jisize*kjsize,.true.) 

      call exec_f_r2c(XgYZ,nx,XYgZ,nxhp,nx,jisize*kjsize)

      if(iproc .gt. 1) then
         call init_f_c(XYgZ,iisize,1,XYZg,iisize,1,ny,iisize,.true.)
         call fcomm1(XYgZ,XYgZ)
         do z=kjstart,kjend
            call ftran_y_zplane(XYgZ,z-kjstart,iisize,kjsize,iisize,1,
     &          XYZg,z-kjstart,iisize,kjsize,iisize,1,ny,iisize)
         enddo
      else
         call init_f_c(XYgZ,nxhp,1,XYZg,nxh,1,ny,nxh,.false.)
         do z=1,kjsize

c In case of 1D decomposition do only FFT in Y without data exchange
            do y=1,ny
               XYgZ(1,y,z) = cmplx(real(XYgZ(1,y,z)),real(XYgZ(nxhp,y,z)))
            enddo
            call ftran_y_zplane(XYgZ,z-1,nxhp,kjsize,nxhp,1,
     &            XYZg,z-1,nxh,kjsize,nxh,1,ny,nxh)
         enddo

      endif

      call clean_x1
      call clean_x2

      if(jproc .gt. 1) then

         call fcomm2(XYZg,XYZg)

c Transform in z for all x and y

         call init_f_c(XYZg,iisize*jjsize, 1,
     &   XYZg,iisize*jjsize, 1,nz,iisize*jjsize,.true.)

         call exec_f_c(XYZg,iisize*jjsize, 1,
     &   XYZg,iisize*jjsize, 1,nz,iisize*jjsize)

      else
c In case of 1D decomposition do only FFT in Y without data exchange
         call init_f_c(XYZg,iisize*ny,1,XYZg,
     &    iisize*ny,1, nz,iisize*ny,.true.)
         call exec_f_c(XYZg,iisize*ny,1,XYZg,
     &    iisize*ny,1, nz,iisize*ny)
      endif

      call clean_x2

      deallocate(XYgZ)

      call free_work

      return
      end subroutine

c Communication Module
c 
c Contains 4 routines for forward and backward exchanges inrows and columns 
c of processors. Uses MPI_Alltoallv routine
c

c========================================================
c Transpose X and Y pencils

      subroutine fcomm1(source,dest)
c========================================================

      implicit none

      complex(mytype) source(nxhp,jistart:jiend,kjsize)
      complex(mytype) dest(iistart:iiend,ny_fft,kjsize)

      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) sendbuf(jisize*nxh)
      integer x,y,i,ierr,z
      integer*8 position

      tp1 = tp1 - MPI_Wtime() 
   
c Pack the real part of nx/2 frequency into the imaginary part of 0 frequency
c (both have imaginary part zero due to conjugte symmetry of a real-to-complex 
c transform)


      do z=1,kjsize
         do y=jistart,jiend
            source(1,y,z) = cmplx(real(source(1,y,z)),real(source(nxhp,y,z)))
         enddo

c Pack the send buffer for exchanging y and x (within a given z plane ) into sendbuf

         position = 1
         do i=0,iproc-1
            do y=jistart,jiend
               do x=iist(i),iien(i)
                  sendbuf(position) = source(x,y,z)
                  position = position +1
               enddo
            enddo
         enddo

         tp1 = tp1 + MPI_Wtime() 
      
         t1 = t1 - MPI_Wtime() 
         
c Exchange the y-x buffers

         call mpi_alltoallv(sendbuf,IfSndCnts, IfSndStrt,mpi_byte, 
     &        dest(iistart,1,z),IfRcvCnts, IfRcvStrt,mpi_byte,mpi_comm_row,ierr)
         t1 = MPI_Wtime() + t1

      enddo

      return
      end subroutine

c========================================================
c Transpose Y and Z pencils

      subroutine fcomm2(source,dest)
c========================================================

      implicit none

      complex(mytype) source(iistart:iiend,ny_fft,kjstart:kjend)
      complex(mytype) dest(iistart:iiend,jjstart:jjend,nz_fft)
      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) sendbuf(iisize*kjsize*ny_fft)
      integer x,z,y,i,ierr
      integer*8 position

c Pack send buffers for exchanging y and z for all x at once 
      position = 1
      t2 = t2 - MPI_Wtime()
 
      do i=0,jproc-1
         do z=kjstart,kjend
            do y=jjst(i),jjen(i)
               do x=iistart,iiend
                  sendbuf(position) = source(x,y,z)
                  position = position+1
               enddo
            enddo
         enddo
      enddo
         
c Exchange y-z buffers

      call mpi_alltoallv(sendbuf,KfSndCnts, KfSndStrt,mpi_byte,
     &     dest,KfRcvCnts, KfRcvStrt,mpi_byte,mpi_comm_col,ierr)
      t2 = MPI_Wtime() + t2
         
      return
      end subroutine


c========================================================
c Transpose back Z to Y pencils

      subroutine bcomm1 (source,dest)
c========================================================

      implicit none

      complex(mytype) source(iistart:iiend,jjstart:jjend,nz_fft)
      complex(mytype) dest(iistart:iiend,ny_fft,kjstart:kjend)

      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) recvbuf1(iisize*kjsize*ny_fft)
      integer x,y,z,i,ierr
      integer*8 position

      t3 = t3 - MPI_Wtime() 
      call mpi_alltoallv(source,JrSndCnts, JrSndStrt,mpi_byte,
     &     recvbuf1,JrRcvCnts, JrRcvStrt,mpi_byte,mpi_comm_col,ierr)
      
c Unpack receive buffers into XZgY

      position=1
      do i=0,jproc-1
         do z=kjstart,kjend
            do y=jjst(i),jjen(i)
               do x=iistart,iiend
                  dest(x,y,z) = recvbuf1(position)
                  position = position+1
               enddo
            enddo
         enddo
      enddo
      
      t3 = t3 + MPI_Wtime() 
      
      return
      end subroutine


c========================================================
c Transpose back Y to X pencils

      subroutine bcomm2(source,dest)
c========================================================

      implicit none

      complex(mytype) dest(nxhp,jistart:jiend,kjstart:kjend)
      complex(mytype) source(iistart:iiend,ny_fft,kjstart:kjend)
      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) recvbuf(jisize*nxh)
      integer x,y,z,i,ierr
      integer*8 position

c Exchange x-z buffers
      
      do z=kjend,kjstart,-1
         t4 = t4 - MPI_Wtime() 
         call mpi_alltoallv (source(iistart,1,z),KrSndCnts, KrSndStrt,
     &      mpi_byte, recvbuf,KrRcvCnts,KrRcvStrt,mpi_byte,   
     &      mpi_comm_row,ierr)
         t4 = t4 + MPI_Wtime() 

c Unpack receive buffers into XgYZ

         position=1
         do i=0,iproc-1
            do y=jistart,jiend
               do x=iist(i),iien(i)
                  dest(x,y,z) = recvbuf(position)
                  position = position+1
               enddo
            enddo
         enddo

c Move the real part of nx/2 frequency to its proper location to prepare
c for the inverse X transform

         do y=jistart,jiend
            dest(nxhp,y,z)=cmplx(imag(dest(1,y,z)),0.0)
            dest(1,y,z) = cmplx(real(dest(1,y,z)),0.0)
         enddo
         
      enddo
      
      return
      end subroutine

      end module







