module com
	
  use p3dfft_var

  implicit none
  
  public

  integer i,m,x,y,z,ierr
  logical, parameter :: flg_inplace = .true.


!-------Grid dimensions--------------------------------------------------
!  integer, parameter :: nx = 2048
!  integer, parameter :: ny = 2048
!  integer, parameter :: nz = 512
!  integer, parameter :: nx = 1024
!  integer, parameter :: ny = 1024
!  integer, parameter :: nz = 256
!  integer, parameter :: nx = 512
!  integer, parameter :: ny = 512
!  integer, parameter :: nz = 128
  integer, parameter :: nx = 256
  integer, parameter :: ny = 256 
  integer, parameter :: nz = 64
  integer, parameter :: nxhp=nx/2+1
  integer, parameter :: nyhp=ny/2+1
  integer, parameter :: nzhp=nz/2+1


!-------Imaginary Unit---------------------------------------------------
  complex, parameter :: ima = (0.0,1.0)


!-------Wavenumber/Geometry----------------------------------------------
  integer, parameter :: ndim = 2
  real(mytype), parameter :: PI = 3.14159265
  real(mytype), parameter :: TWOPI = 2.0*PI
  
  real(mytype), parameter :: Lx = 2.0*PI
  real(mytype), parameter :: Ly = 2.0*PI
  real(mytype), parameter :: Lz = 2.0*PI
  integer,      parameter :: asp= 4            !Aspect ratio = 2*asp

  real(mytype) :: ax(1:nx), ay(1:ny), az(1:nz) 
  real(mytype) :: kx(1:nx/2+1), ky(1:ny), kz(1:nz)

!-------Normalization factor---------------------------------------------
  real(8) fak
	
!-------Velocity field parameters----------------------------------------
  real(mytype), parameter ::  R_inv =1./400./3.33   !Viscosity (~inverse Reynolds number)
  real(mytype), parameter ::  S=0.0            !Shear rate for the flow = 2*S/pi
  real(mytype), parameter ::  epsin=0.0        !Energy injection rate
  real(mytype), parameter ::  Om=2.0           !Angular frequency Om*e_z


#ifdef SCALAR
!-------Scalar field parameters------------------------------------------
  real(mytype), parameter ::  Pa_inv =1./280./3.33     !Diffusivity (~inverse Peclet number)
  real(mytype), parameter ::  Pra = R_inv/Pa_inv  !Prandtl number
  real(mytype), parameter ::  Ga=pi               !Scalar mean gradient =2*G/pi
#endif


!-------Time advancement-------------------------------------------------
#ifdef FIXED
  integer :: istep,nsamp
  integer, parameter :: nstep=20
  integer, parameter :: iout=2500
  real(mytype), parameter :: dt=0.001
  real(mytype) :: dt2,time,velmax,cfl
#endif
#ifdef ADAPT
  integer :: ioflag,nsamp
  real(mytype) :: dt,dt2,time,velmax,cfl
  real(mytype) :: dtout,tout,tend
  real(mytype), parameter :: cfl=0.5
#endif


!-------Integrating factors----------------------------------------------
  real(mytype) :: xyfac(1:nx/2+1,1:ny)
  real(mytype) :: zfac(1:nz)

#ifdef SCALAR
  real(mytype) :: xydifa(1:nx/2+1,1:ny)
  real(mytype) :: zdifa(1:nz)
#endif


!-------De-aliasing/phase shifting---------------------------------------
        real mask(1:nx/2+1,1:ny)


contains

!-----------------------------------------------------------------
  subroutine init_axis

    do z=1,nz
       az(z)=Lz*(z-1)/nz
    enddo
    do y=1,ny
       ay(y)=Ly*(y-1)/ny
    enddo
    do x=1,nx
       ax(x)=Lx*(x-1)/nx
    enddo
    
  end subroutine init_axis
  
!----------------------------------------------------------------  
  subroutine init_wave
    
    do z=1,nz
       kz(z)=twopi*real(z-1-z/(nz/2+2)*nz)/Lz
    enddo
        
    do y=1,ny
       ky(y)=twopi*real(y-1-y/(ny/2+2)*ny)/Ly
    enddo

    do x=1,nxhp
       kx(x)=twopi*real(x-1)/Lx
    enddo

  end subroutine init_wave

end module com
