program fft3d

  use p3dfft_var
  use com

  implicit none

  real(8) inittime,starttime,endtime
  integer dims(2),istep
  integer ipic

!---Fields
  real(mytype), dimension(:,:,:), allocatable :: ux,uy,uz
  real(mytype), dimension(:,:,:), allocatable :: Fux,Fuy
  real(mytype), dimension(:,:,:), allocatable :: uxtt,uytt
  real(mytype), dimension(:,:,:), allocatable :: rij
#ifdef SCALAR
  real(mytype), dimension(:,:,:), allocatable :: tha
  real(mytype), dimension(:,:,:), allocatable :: Ftha
  real(mytype), dimension(:,:,:), allocatable :: thatt
#endif

  nx_fft = nx
  ny_fft = ny
  nz_fft = nz

  call MPI_INIT (ierr)

  inittime = MPI_wtime()

  call MPI_COMM_SIZE (MPI_COMM_WORLD,numtasks,ierr)
  call MPI_COMM_RANK (MPI_COMM_WORLD,taskid,ierr)

  !---numtasks is devided into a iproc x jproc stencle
  dims(1) = 0
  dims(2) = 0
  call MPI_Dims_create(numtasks,2,dims,ierr)
!  if(dims(1) .gt. dims(2)) then
!     dims(1) = dims(2)
!     dims(2) = numtasks / dims(1)
!  endif

!  dims(1) = 64
!  dims(2) = 16 

  iproc = dims(1)
  jproc = dims(2)

  if(taskid.eq.0)write(*,*)'iproc =',iproc,', jproc =',jproc

  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
  
!---Initialize the processor grid
  call p3dfft_setup (dims,nx,ny,nz)
  
!---Initialize Geometry/Wavenumbers 
  call init_axis
  call init_wave

!---Number of snapshot 
  ipic=0
  
!---Allocate fields
  allocate(ux(nx,jisize,kjsize))
  allocate(uy(nx,jisize,kjsize))
  allocate(uz(nx,jisize,kjsize))

  allocate(Fux(nx,jisize,kjsize))
  allocate(Fuy(nx,jisize,kjsize))

  allocate(uxtt(nx,jisize,kjsize))
  allocate(uytt(nx,jisize,kjsize))

  allocate(rij(nx,jisize,kjsize))

#ifdef SCALAR
  allocate(tha(nx,jisize,kjsize))
  allocate(Ftha(nx,jisize,kjsize))
  allocate(thatt(nx,jisize,kjsize))
#endif

!---Initialize fields----------------------------------- 
#ifdef SCALAR
  call init_r(ux,uy,uz,tha)
#else
  call init_r(ux,uy,uz)
#endif

  fak=1.0/(dble(nx)*dble(ny)*dble(nz))
  if(taskid.eq.0)write(*,*)'fak',fak,real(fak)

!---Transformation to Fourier space
  call p3dfft_ftran_r2c (ux,ux,flg_inplace)
  call p3dfft_ftran_r2c (uy,uy,flg_inplace)
#ifdef SCALAR
  call p3dfft_ftran_r2c (tha,tha,flg_inplace)
#endif	

!---Normalization of Fourier spectrum
  ux=ux*real(fak)
  uy=uy*real(fak)
#ifdef SCALAR
  tha=tha*real(fak)
#endif

!---Incompressibility/Enforce symmetry at kx=0
#ifdef SCALAR
  call symm(ux,uy,uz,tha)
#else
  call symm(ux,uy,uz)
#endif

!---Check div(u)=0 
  call divch(ux,uy,uz)

  starttime = MPI_wtime()

!---Begin integration loop-----------------------------
  DO istep=1,nstep

!---Predictor step
#ifdef SCALAR
    call predic(ux,uy,uz,tha,Fux,Fuy,Ftha,uxtt,uytt,thatt,rij)

    if(taskid.eq.0)then
     ux(1,1,1)=(0.,0.)
     uy(1,1,1)=(0.,0.)
     uz(1,1,1)=(0.,0.)
     tha(1,1,1)=(0.,0.)
    endif

    call symm(ux,uy,uz,tha)
#else
    call predic(ux,uy,uz,Fux,Fuy,uxtt,uytt,rij)

    if(taskid.eq.0)then
     ux(1,1,1)=(0.,0.)
     uy(1,1,1)=(0.,0.)
     uz(1,1,1)=(0.,0.)
    endif

    call symm(ux,uy,uz)
#endif

!---Corrector step
#ifdef SCALAR
    call correc(ux,uy,uz,tha,Fux,Fuy,Ftha,uxtt,uytt,thatt,rij)

    if(taskid.eq.0)then
     ux(1,1,1)=(0.,0.)
     uy(1,1,1)=(0.,0.)
     uz(1,1,1)=(0.,0.)
     tha(1,1,1)=(0.,0.)
    endif

    call symm(ux,uy,uz,tha)
#else
    call correc(ux,uy,uz,Fux,Fuy,uxtt,uytt,rij)

    if(taskid.eq.0)then
     ux(1,1,1)=(0.,0.)
     uy(1,1,1)=(0.,0.)
     uz(1,1,1)=(0.,0.)
    endif

    call symm(ux,uy,uz)
#endif

!---Analysis output
    if((istep/20)*20==istep)then 
#ifdef SCALAR
      call v_ene(ux,uy,uz,tha)
      call v_eps(ux,uy,uz,tha)
      call divch(ux,uy,uz)
#else
      call v_ene(ux,uy,uz)
      call v_eps(ux,uy,uz)
      call divch(ux,uy,uz)
#endif
    endif

!---Data output
    if((istep/iout)*iout==istep)then
#ifdef SCALAR
     call v_outk1(ux,uy,uz,tha,ipic)
#else
     call v_outk1(ux,uy,uz,ipic)
#endif
     ipic=ipic+1
    endif

  ENDDO 
!---End integration loop---------------------------------------


  endtime = MPI_wtime()
 
  call MPI_FINALIZE (ierr)
  
  if (taskid.eq.0) then 
     write(*,*)'time is',endtime-starttime,'seconds'
     write(*,*)'Duration % MPI_Init/Finalize =',endtime-inittime,'seconds'
  endif

end program fft3d

!=============================================================
subroutine print_all_k(A)

  use p3dfft_var
  use com

  implicit none
  
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: A
  
!  A=A/nx/ny/nz
  
  write(*,*)'(Fourier) taskid',taskid,' x:',iistart,iiend,' y:',jjstart,jjend
  
  call MPI_Barrier(MPI_COMM_WORLD,ierr)
  
  do z=1,nz
     do y=jjstart,jjend
        do x=iistart,iiend
	   if(abs(A(x,y,z)) .gt. 1.e-5) then
              write(*,*)taskid,x,y,z,A(x,y,z)
	   endif
        enddo
     enddo
  enddo
  
end subroutine print_all_k

!------------------------------------------------------------------
#ifdef SCALAR
subroutine init_r(ux_r,uy_r,uz_r,tha_r)
#else
subroutine init_r(ux_r,uy_r,uz_r)
#endif
    
  use p3dfft_var
  use com
  
  implicit none
    
  real(mytype), dimension(1:nx,jistart:jiend,kjstart:kjend) :: ux_r,uy_r,uz_r
#ifdef SCALAR
  real(mytype), dimension(1:nx,jistart:jiend,kjstart:kjend) :: tha_r
#endif

!---Initialization
  do z=kjstart,kjend
     do y=jistart,jiend
        do x=1,nx
           ux_r(x,y,z)= cos(kz(2)*az(z))*sin(ky(9)*ay(y))    
           uy_r(x,y,z)= cos(kz(2)*az(z))*cos(kx(3)*ax(x))   
           uz_r(x,y,z)= cos(kx(9)*ax(x))
           
#ifdef SCALAR
           tha_r(x,y,z)=sin(kz(2)*az(z))*sin(ky(3)*ay(y))   
#endif
        enddo
     enddo
  enddo
  
end subroutine init_r
 
!------------------------------------------------------------------ 
#ifdef SCALAR
subroutine compare_r(ux_r,uy_r,uz_r,tha_r,ux0_r,uy0_r,uz0_r,tha0_r)
#else
subroutine compare_r(ux_r,uy_r,uz_r,ux0_r,uy0_r,uz0_r)
#endif

  use p3dfft_var
  use com
 
  implicit none

  real(mytype), dimension(1:nx,jistart:jiend,kjstart:kjend) :: ux_r,uy_r,uz_r,ux0_r,uy0_r,uz0_r
#ifdef SCALAR
  real(mytype), dimension(1:nx,jistart:jiend,kjstart:kjend) :: tha_r,tha0_r
#endif

  real(mytype) :: maxx,maxy,maxz,maxt

!---Difference after forward/backward
  maxx=maxval(abs(ux0_r(:,:,:)-ux_r(:,:,:)))
  maxy=maxval(abs(uy0_r(:,:,:)-uy_r(:,:,:)))
  maxz=maxval(abs(uz0_r(:,:,:)-uz_r(:,:,:)))
#ifdef SCALAR
  maxt=maxval(abs(tha0_r(:,:,:)-tha_r(:,:,:)))
#endif
 
  if (maxx.ge.1e-5) then
    write(*,*) taskid,'maxx',maxx
  end if
  if (maxy.ge.1e-5) then
    write(*,*) taskid,'maxy',maxy
  end if
  if (maxz.ge.1e-5) then
    write(*,*) taskid,'maxz',maxz
  end if
#ifdef SCALAR
  if (maxt.ge.1e-5) then
    write(*,*) taskid,'maxt',maxt
  end if
#endif

end subroutine compare_r
