#ifdef SCALAR
subroutine predic(ux_k,uy_k,uz_k,tha_k,F_ux_k,F_uy_k,F_tha_k,ux_ktemp,uy_ktemp,tha_ktemp,uij_k)
#else
subroutine predic(ux_k,uy_k,uz_k,F_ux_k,F_uy_k,ux_ktemp,uy_ktemp,uij_k)
#endif

  use com

  implicit none

  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: ux_k,uy_k,uz_k
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: ux_ktemp,uy_ktemp
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: F_ux_k,F_uy_k
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: uij_k
#ifdef SCALAR
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: tha_k
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: tha_ktemp
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: F_tha_k
#endif

  real(mytype) :: fact(1:nxhp)
  real(mytype) :: fact1a(1:nxhp)

!-------Shift the Fourier fields to temporary arrays
  ux_ktemp(:,:,:)=ux_k(:,:,:)
  uy_ktemp(:,:,:)=uy_k(:,:,:)
#ifdef SCALAR
  tha_ktemp(:,:,:)=tha_k(:,:,:)
#endif

      
!-------Rate of change 
#ifdef SCALAR
  call rhs_sc(ux_k,uy_k,uz_k,tha_k,F_ux_k,F_uy_k,F_tha_k,uij_k)
#else
  call rhs_sc(ux_k,uy_k,uz_k,F_ux_k,F_uy_k,uij_k)
#endif

!-------Evaluate the new time step
  call step(ux_k,uy_k,uz_k)

!-------Transform back to Fourier-------------------------
  call p3dfft_ftran_r2c (ux_k,ux_k,flg_inplace)
  call p3dfft_ftran_r2c (uy_k,uy_k,flg_inplace)	

  ux_k=ux_k*real(fak)
  uy_k=uy_k*real(fak)

#ifdef SCALAR
  call p3dfft_ftran_r2c (tha_k,tha_k,flg_inplace)

  tha_k=tha_k*real(fak)
#endif

!-------Predictor step	
  do z=1,nzhp

    call masks(z)

    do y=jjstart,jjend 

      do x=iistart,iiend

!----------Integrating factor
	fact(x)=xyfac(x,y)*zfac(z)

	ux_k(x,y,z)=(ux_ktemp(x,y,z)+dt*F_ux_k(x,y,z))* &
	            fact(x)*mask(x,y)
	uy_k(x,y,z)=(uy_ktemp(x,y,z)+dt*F_uy_k(x,y,z))* &
	            fact(x)*mask(x,y)	   
#ifdef SCALAR
	fact1a(x)=xydifa(x,y)*zdifa(z)	   
	   	   
	tha_k(x,y,z)=(tha_ktemp(x,y,z)+dt*F_tha_k(x,y,z))* &
	             fact1a(x)*mask(x,y)
#endif	   

!----------Temporary results necessary for the corrector step
        ux_ktemp(x,y,z)=(ux_ktemp(x,y,z)+dt2*F_ux_k(x,y,z))* &
                        fact(x)*mask(x,y)
        uy_ktemp(x,y,z)=(uy_ktemp(x,y,z)+dt2*F_uy_k(x,y,z))* &
                        fact(x)*mask(x,y)
#ifdef SCALAR
        tha_ktemp(x,y,z)=(tha_ktemp(x,y,z)+dt2*F_tha_k(x,y,z))* &
                         fact1a(x)*mask(x,y)
#endif     

      enddo
    enddo  
  enddo  

end subroutine predic
