#ifdef SCALAR
subroutine rhs_sc(ux_k,uy_k,uz_k,tha_k,F_ux_k,F_uy_k,F_tha_k,uij_k)
#else
subroutine rhs_sc(ux_k,uy_k,uz_k,F_ux_k,F_uy_k,uij_k)
#endif

  use com

  implicit none

  real(mytype) :: fak1,fakg
  real(mytype) :: kxsq,kysq,kzsq,ksq
  complex(mytype) :: prod1,prod2,prod3

  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: ux_k,uy_k,uz_k
#ifdef SCALAR
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: tha_k
#endif

  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: F_ux_k,F_uy_k
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: uij_k
#ifdef SCALAR
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: F_tha_k
#endif
	
#ifdef REWA
  integer :: kro(iistart:iiend,jjstart:jjend,1:nz)
#endif


#ifdef SCALAR
!-------Mean gradient forcing (!!! take slope now over 2pi !!!!) 
  F_tha_k(:,:,:)=(0.,0.)
  F_tha_k(:,:,:)=Ga/pi*uz_k(:,:,:)
#endif

  F_ux_k(:,:,:)=(0.,0.)
  F_uy_k(:,:,:)=(0.,0.)



#ifdef REWA
!-------Reduced waveset forcing-----------------------------
  kro(:,:,:)=0

  fak1=0.
  fakg=0.

  if(taskid.eq.0)then
    kro(asp*1+1,   asp*2+1,3)=1 ! (kx= 1,ky= 2,kz= 2)  
    kro(asp*1+1,ny-asp*2+1,3)=1 ! (kx= 1,ky=-2,kz= 2)
    kro(asp*2+1,   asp*1+1,3)=1 ! (kx= 2,ky= 1,kz= 2)  
    kro(asp*2+1,ny-asp*1+1,3)=1 ! (kx= 2,ky=-1,kz= 2)
    kro(asp*2+1,   asp*2+1,2)=1 ! (kx= 2,ky= 2,kz= 1)  
    kro(asp*2+1,ny-asp*2+1,2)=1 ! (kx= 2,ky=-2,kz= 1)

    kro(asp*2+1,   asp*1+1,2)=1 ! (kx= 2,ky= 1,kz= 1)  
    kro(asp*2+1,ny-asp*1+1,2)=1 ! (kx= 2,ky=-1,kz= 1)
    kro(asp*1+1,   asp*2+1,2)=1 ! (kx= 1,ky= 2,kz= 1)  
    kro(asp*1+1,ny-asp*2+1,2)=1 ! (kx= 1,ky=-2,kz= 1)
    kro(asp*1+1,   asp*1+1,3)=1 ! (kx= 1,ky= 1,kz= 2)  
    kro(asp*1+1,ny-asp*1+1,3)=1 ! (kx= 1,ky=-1,kz= 2)
  endif

  if(taskid.eq.jproc-1)then
    kro(asp*1+1,   asp*2+1,nz-1)=1 ! (kx= 1,ky= 2,kz=-2)  
    kro(asp*1+1,ny-asp*2+1,nz-1)=1 ! (kx= 1,ky=-2,kz=-2)
    kro(asp*2+1,   asp*1+1,nz-1)=1 ! (kx= 2,ky= 1,kz=-2)  
    kro(asp*2+1,ny-asp*1+1,nz-1)=1 ! (kx= 2,ky=-1,kz=-2)
    kro(asp*2+1,   asp*2+1,nz-0)=1 ! (kx= 2,ky= 2,kz=-1)  
    kro(asp*2+1,ny-asp*2+1,nz-0)=1 ! (kx= 2,ky=-2,kz=-1)

    kro(asp*2+1,   asp*1+1,nz-0)=1 ! (kx= 2,ky= 1,kz=-1)  
    kro(asp*2+1,ny-asp*1+1,nz-0)=1 ! (kx= 2,ky=-1,kz=-1)
    kro(asp*1+1,   asp*2+1,nz-0)=1 ! (kx= 1,ky= 2,kz=-1)  
    kro(asp*1+1,ny-asp*2+1,nz-0)=1 ! (kx= 1,ky=-2,kz=-1)
    kro(asp*1+1,   asp*1+1,nz-1)=1 ! (kx= 1,ky= 1,kz=-2)   
    kro(asp*1+1,ny-asp*1+1,nz-1)=1 ! (kx= 1,ky=-1,kz=-2)
  endif

  do z=1,nz
    do y=jjstart,jjend
      do x=iistart,iiend
        fak1=fak1+kro(x,y,z)*ux_k(x,y,z)*conjg(ux_k(x,y,z))+&
	        kro(x,y,z)*uy_k(x,y,z)*conjg(uy_k(x,y,z))+&
	        kro(x,y,z)*uz_k(x,y,z)*conjg(uz_k(x,y,z))
      enddo
    enddo
  enddo
  fak1=2*fak1 ! Reality condition  

  call MPI_REDUCE (fak1,fakg,1,MPI_REAL,MPI_SUM,0,MPI_COMM_WORLD,mpierr)
  call MPI_BCAST (fakg,1,MPI_REAL,0,MPI_COMM_WORLD,mpierr)

  do z=1,nz
    if(z.eq.1)goto 1000  ! do not divide by kz=0
    do y=jjstart,jjend
      do x=iistart,iiend
        F_ux_k(x,y,z)=kro(x,y,z)*epsin*ux_k(x,y,z)/fakg
        F_uy_k(x,y,z)=kro(x,y,z)*epsin*uy_k(x,y,z)/fakg
      enddo
    enddo
1000 continue      
  enddo
	
#endif



#ifdef SHEAR
!-------Laminar profile sustainance--------------
!       Can be combined with Reduced waveset forcing
!       Note: update goes over half the box only
!------------------------------------------------
  if (taskid.eq.0)F_ux_k(1,2,1)=F_ux_k(1,2,1)-S*R_inv/2.
#endif



!---------------------------------------------------------------------
#ifdef CONV
!-------Buoyancy term plus incompressibility correction
!       Can be combined with shear forcing
!---------------------------------------------------------------------
  IF (taskid.eq.0) THEN 

  do z=1,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart,jjend
      kysq=ky(y)*ky(y)

	do x=iistart+1,iiend
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)-kx(x)*kz(z)*tha_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-ky(y)*kz(z)*tha_k(x,y,z)/ksq

      enddo
    enddo
  enddo

  do z=1,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart+1,jjend
      kysq=ky(y)*ky(y)

	do x=iistart,iistart
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)-kx(x)*kz(z)*tha_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-ky(y)*kz(z)*tha_k(x,y,z)/ksq

      enddo
    enddo
  enddo

  do z=2,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart,jjstart
      kysq=ky(y)*ky(y)

	do x=iistart,iistart
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)-kx(x)*kz(z)*tha_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-ky(y)*kz(z)*tha_k(x,y,z)/ksq

      enddo
    enddo
  enddo


  ELSE

  do z=1,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart,jjend
      kysq=ky(y)*ky(y)

	do x=iistart,iiend
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)-kx(x)*kz(z)*tha_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-ky(y)*kz(z)*tha_k(x,y,z)/ksq

      enddo
    enddo
  enddo

  ENDIF

#endif



!---------------------------------------------------------------------
#ifdef ROTATION
!-------Coriolis term plus incompressibility correction
!       Can be combined with shear forcing and convection
!---------------------------------------------------------------------
  IF (taskid.eq.0) THEN 

  do z=1,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart,jjend
      kysq=ky(y)*ky(y)

	do x=iistart+1,iiend
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)+2*(kysq+kzsq)*Om*uy_k(x,y,z)/ksq &
	                              +2*kx(x)*ky(y)*Om*ux_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-2*kx(x)*ky(y)*Om*uy_k(x,y,z)/ksq &
	                              -2*(kxsq+kzsq)*Om*ux_k(x,y,z)/ksq

      enddo
    enddo
  enddo

  do z=1,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart+1,jjend
      kysq=ky(y)*ky(y)

	do x=iistart,iistart
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)+2*(kysq+kzsq)*Om*uy_k(x,y,z)/ksq &
	                              +2*kx(x)*ky(y)*Om*ux_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-2*kx(x)*ky(y)*Om*uy_k(x,y,z)/ksq &
	                              -2*(kxsq+kzsq)*Om*ux_k(x,y,z)/ksq

      enddo
    enddo
  enddo

  do z=2,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart,jjstart
      kysq=ky(y)*ky(y)

	do x=iistart,iistart
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)+2*(kysq+kzsq)*Om*uy_k(x,y,z)/ksq &
	                              +2*kx(x)*ky(y)*Om*ux_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-2*kx(x)*ky(y)*Om*uy_k(x,y,z)/ksq &
	                              -2*(kxsq+kzsq)*Om*ux_k(x,y,z)/ksq

      enddo
    enddo
  enddo


  ELSE

  do z=1,nzhp              ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)
	 
    do y=jjstart,jjend
      kysq=ky(y)*ky(y)

	do x=iistart,iiend
	   kxsq=kx(x)*kx(x)
	   ksq=kxsq+kysq+kzsq

!----------x direction (lateral)
	   F_ux_k(x,y,z)=F_ux_k(x,y,z)+2*(kysq+kzsq)*Om*uy_k(x,y,z)/ksq &
	                              +2*kx(x)*ky(y)*Om*ux_k(x,y,z)/ksq

!----------y direction (lateral)
	   F_uy_k(x,y,z)=F_uy_k(x,y,z)-2*kx(x)*ky(y)*Om*uy_k(x,y,z)/ksq &
	                              -2*(kxsq+kzsq)*Om*ux_k(x,y,z)/ksq

      enddo
    enddo
  enddo

  ENDIF

#endif






!-------FFT to physical space
  call p3dfft_btran_c2r (ux_k,ux_k,flg_inplace) 
  call p3dfft_btran_c2r (uy_k,uy_k,flg_inplace) 
  call p3dfft_btran_c2r (uz_k,uz_k,flg_inplace) 
#ifdef SCALAR
  call p3dfft_btran_c2r (tha_k,tha_k,flg_inplace) 
#endif

!-------u_x*u_x-----------------------------------------------------
  call reynolds (ux_k,ux_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint

!-------Right hand side of NSE
  IF (taskid.eq.0) THEN 

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart+1,iiend
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       -ima*(kysq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq 
		
      enddo
    enddo
  enddo

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart+1,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       -ima*(kysq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq 
		
      enddo
    enddo
  enddo

  do z=2,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjstart       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       -ima*(kysq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq 
		
      enddo
    enddo
  enddo

  ELSE 

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iiend
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       -ima*(kysq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq 
		
      enddo
    enddo
  enddo

  ENDIF 



!-------u_x*u_y-----------------------------------------------------
  call reynolds (ux_k,uy_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint

!-------Right hand side of NSE
  IF (taskid.eq.0) THEN
 
  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart+1,iiend
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq        &
		               -ima*(kxsq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq     
		
      enddo
    enddo
  enddo

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart+1,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq        &
		               -ima*(kxsq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq     
		
      enddo
    enddo
  enddo

  do z=2,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjstart       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq        &
		               -ima*(kxsq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq     
		
      enddo
    enddo
  enddo
 
  ELSE

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iiend
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kxsq*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq        &
		               -ima*(kxsq+kzsq)*kx(x)*fak*uij_k(x,y,z)/ksq     
		
      enddo
    enddo
  enddo
  
  ENDIF



!-------u_x*u_z-----------------------------------------------------
  call reynolds (ux_k,uz_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint
  do y=jjstart,jjend
    do x=iistart,iiend
      uij_k(x,y,1)=(0.,0.)
    enddo
  enddo 

!-------Right hand side of NSE
  IF (taskid.eq.0) THEN

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart+1,iiend
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*kz(z)*kxsq*fak*uij_k(x,y,z)/ksq           
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart+1,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*kz(z)*kxsq*fak*uij_k(x,y,z)/ksq           
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  do z=2,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjstart       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*kz(z)*kxsq*fak*uij_k(x,y,z)/ksq           
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  ELSE

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iiend
	kxsq=kx(x)*kx(x)
	ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       -ima*(kysq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*kz(z)*kxsq*fak*uij_k(x,y,z)/ksq           
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq &
		               +ima*ky(y)*kx(x)*kz(z)*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  ENDIF



!-------u_y*u_y-----------------------------------------------------
  call reynolds (uy_k,uy_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint

!-------Right hand side of NSE
  IF (taskid.eq.0) THEN

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart+1,iiend
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart+1,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  do z=2,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjstart       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq    

      enddo
    enddo
  enddo

  ELSE

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iiend
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*kysq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*ky(y)*fak*uij_k(x,y,z)/ksq    

      enddo
    enddo
  enddo

  ENDIF



!-------u_y*u_z-----------------------------------------------------
  call reynolds (uy_k,uz_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint
  do y=jjstart,jjend
    do x=iistart,iiend
      uij_k(x,y,1)=(0.,0.)
    enddo
  enddo 

!-------Right hand side of NSE
  IF (taskid.eq.0) THEN

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend  
      kysq=ky(y)*ky(y)

      do x=iistart+1,iiend
	kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kz(z)*kysq*fak*uij_k(x,y,z)/ksq    

      enddo
    enddo
  enddo

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart+1,jjend  
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kz(z)*kysq*fak*uij_k(x,y,z)/ksq    

      enddo
    enddo
  enddo

  do z=2,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjstart  
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
	kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kz(z)*kysq*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  ELSE
 
  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend  
      kysq=ky(y)*ky(y)

      do x=iistart,iiend
	kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq

!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                           &
	                       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kx(x)*ky(y)*kz(z)*fak*uij_k(x,y,z)/ksq    
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                           &
	                       -ima*(kxsq+kzsq)*kz(z)*fak*uij_k(x,y,z)/ksq & 
			       +ima*kz(z)*kysq*fak*uij_k(x,y,z)/ksq    

      enddo
    enddo
  enddo

  ENDIF


!-------u_z*u_z-----------------------------------------------------
  call reynolds (uz_k,uz_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint

!-------Right hand side of NSE
  IF (taskid.eq.0) THEN

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart+1,iiend
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       +ima*kx(x)*kzsq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kzsq*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart+1,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       +ima*kx(x)*kzsq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kzsq*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  do z=2,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjstart       
      kysq=ky(y)*ky(y)

      do x=iistart,iistart
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       +ima*kx(x)*kzsq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kzsq*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  ELSE

  do z=1,nzhp                 ! Only half of the loop due to symmetry
    kzsq=kz(z)*kz(z)

    do y=jjstart,jjend       
      kysq=ky(y)*ky(y)

      do x=iistart,iiend
        kxsq=kx(x)*kx(x)
        ksq=kxsq+kysq+kzsq
	
!---------------F_ux_k		
		F_ux_k(x,y,z)=F_ux_k(x,y,z)                          &
	                       +ima*kx(x)*kzsq*fak*uij_k(x,y,z)/ksq     
				
!---------------F_uy_k		
		F_uy_k(x,y,z)=F_uy_k(x,y,z)                          &
	                       +ima*ky(y)*kzsq*fak*uij_k(x,y,z)/ksq    
		
      enddo
    enddo
  enddo

  ENDIF


#ifdef SCALAR
!-------u_x*tha-----------------------------------------------------
  call reynolds (ux_k,tha_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint
  do y=jjstart,jjend
    do x=iistart,iiend
      uij_k(x,y,1)=(0.,0.)
    enddo
  enddo

!-------Right hand side of ADE
  IF (taskid.eq.0) THEN

  do z=1,nzhp
    do y=jjstart,jjend       
      do x=iistart+1,iiend
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kx(x)*fak*uij_k(x,y,z)
      enddo
    enddo
  enddo

  do z=1,nzhp
    do y=jjstart+1,jjend       
      do x=iistart,iistart
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kx(x)*fak*uij_k(x,y,z)
      enddo
    enddo
  enddo

  do z=2,nzhp
    do y=jjstart,jjstart       
      do x=iistart,iistart
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kx(x)*fak*uij_k(x,y,z)
      enddo
    enddo
  enddo

  ELSE

  do z=1,nzhp
    do y=jjstart,jjend       
      do x=iistart,iiend
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kx(x)*fak*uij_k(x,y,z)
      enddo
    enddo
  enddo

  ENDIF



!-------u_y*tha-----------------------------------------------------
  call reynolds (uy_k,tha_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint
  do y=jjstart,jjend
    do x=iistart,iiend
      uij_k(x,y,1)=(0.,0.)
    enddo
  enddo 

!-------Right hand side of ADE
  IF (taskid.eq.0) THEN

  do z=1,nzhp
    do y=jjstart,jjend     
      do x=iistart+1,iiend
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*ky(y)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  do z=1,nzhp
    do y=jjstart+1,jjend     
      do x=iistart,iistart
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*ky(y)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  do z=2,nzhp
    do y=jjstart,jjstart     
      do x=iistart,iistart
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*ky(y)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  ELSE

  do z=1,nzhp
    do y=jjstart,jjend     
      do x=iistart,iiend
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*ky(y)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  ENDIF



!-------u_z*tha-----------------------------------------------------
  call reynolds (uz_k,tha_k,uij_k)
  call p3dfft_ftran_r2c (uij_k,uij_k,flg_inplace)

!-------Apply symmetry constraint

!-------Right hand side of ADE
  IF (taskid.eq.0) THEN

  do z=1,nzhp
    do y=jjstart,jjend      
      do x=iistart+1,iiend
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kz(z)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  do z=1,nzhp
    do y=jjstart+1,jjend      
      do x=iistart,iistart
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kz(z)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  do z=2,nzhp
    do y=jjstart,jjstart      
      do x=iistart,iistart
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kz(z)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  ELSE

  do z=1,nzhp
    do y=jjstart,jjend      
      do x=iistart,iiend
        F_tha_k(x,y,z)=F_tha_k(x,y,z)-ima*kz(z)*fak*uij_k(x,y,z)  
      enddo
    enddo
  enddo

  ENDIF		
#endif
	end





!----------------------------------------------------------------------
subroutine reynolds(A_r,B_r,uij_r)

  use p3dfft_var
  use com

  implicit none

  real(mytype), dimension(1:nx,jistart:jiend,kjstart:kjend) :: A_r,B_r,uij_r
  uij_r(:,:,:)=0.

  do z=kjstart,kjend
    do y=jistart,jiend
      do x=1,nx 
        uij_r(x,y,z)=A_r(x,y,z)*B_r(x,y,z)
      enddo
    enddo
  enddo

end subroutine reynolds
