subroutine step(ux_r,uy_r,uz_r)
!---------------------------------------------------------------------
!       Up to 4 different scalars can be advected
!---------------------------------------------------------------------	

  use p3dfft_var
  use com

  implicit none


  real(mytype) :: logx(1:nxhp)
  real(mytype) :: logy(1:ny)
  real(mytype) :: logz(1:nz)
  real(mytype) :: term,t1,t2
  real(mytype) :: vmaxz(1:numtasks)

  real(mytype), dimension(1:nx,jistart:jiend,kjstart:kjend) :: ux_r,uy_r,uz_r

  real vel
  integer ierror,ierrcode,iostep,jstep
 
!-------Courant criterion----------------------------------------
  velmax=0.
  do z=kjstart,kjend
    do y=jistart,jiend
      do x=1,nx
	vel=abs(ux_r(x,y,z))+abs(uy_r(x,y,z))+abs(uz_r(x,y,z))
	velmax=amax1(vel,velmax)
      enddo
    enddo
  enddo

  call MPI_ALLGATHER (velmax,1,MPI_REAL,vmaxz,1,MPI_REAL,MPI_COMM_WORLD,ierr)

  velmax=0.
  do z=1,numtasks
    velmax=amax1(velmax,vmaxz(z))
  enddo
	
#ifdef FIXED
!-------Monitoring the Courant number	
  cfl=dt*nz*velmax/Lz	
  if(taskid.eq.0)write(40,*)time,velmax,cfl
	
!-------Increment the time
  t1=time
  time=time+dt
  t2=time
#endif
#ifdef ADAPT
!-------Adaptive step 
  dt=cfl*Ly/(velmax*ny)
                  
!-------Increment the time
  t1=time
  time=time+dt
  t2=time

!-------Is tout or tend reached?        
  if(t2.gt.tout)then
    dt=tout-t1
    time=t1+dt               ! Take t1 for incrementing
    tout=tout+dtout
    t2=time
    ioflag=1
  endif
#endif

  dt2=dt/2.   ! half step for corrector routine

!-------Is the step too small/large?
  if (dt.gt.10..or.dt.lt.1.e-6) then
    write (6,*) 'dt gotten out of control in sub. step, dt=',dt
    call MPI_ABORT (MPI_COMM_WORLD,ierrcode,ierror)
  end if


!-------Integrating factor 
  do x=1,nxhp
    logx(x)=-R_inv*kx(x)**2 * dt
  enddo
	
  do y=1,ny
   logy(y)=-R_inv*ky(y)**2 * dt
  enddo
	
  do z=1,nz
   logz(z)=-R_inv*kz(z)**2 * dt
  enddo
	
  do y=1,ny
    do x=1,nxhp
     term=logx(x)+logy(y)
     xyfac(x,y)=exp(term)
	  
#ifdef SCALAR	  
     xydifa(x,y)=exp(term/Pra)
#endif	  
    enddo
  enddo
	
  do z=1,nz
    zfac(z)=exp(logz(z))
         
#ifdef SCALAR	  
    zdifa(z)=exp(logz(z)/Pra)
#endif	  
  enddo

end subroutine step
	

