#ifdef SCALAR
subroutine symm(ux_k,uy_k,uz_k,tha_k)
#else
subroutine symm(ux_k,uy_k,uz_k)
#endif

  use p3dfft_var
  use com
	
  implicit none

  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: ux_k,uy_k,uz_k
#ifdef SCALAR
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: tha_k
#endif

!-------Incompressibility of the flow---------------------
  uz_k(:,:,:)=(0.,0.)

!-------Case kz=0
  do z=1,1
    do y=jjstart,jjend
      do x=iistart,iiend
        if(x.gt.1) ux_k(x,y,z)=-ky(y)*uy_k(x,y,z)/kx(x)
      enddo
    enddo
  enddo

!-------Case kz ne 0
  do z=2,nz
    do y=jjstart,jjend
      do x=iistart,iiend
        uz_k(x,y,z)=(-kx(x)*ux_k(x,y,z)-ky(y)*uy_k(x,y,z))/kz(z)
      enddo
    enddo
  enddo

!-------Enforce conjugate symmetry at kx=0
  call enfsymp(ux_k)
  call enfsymp(uy_k)
  call enfsymm(uz_k)
#ifdef SCALAR
  call enfsymm_th(tha_k)
#endif
	
!-------Apply symmetry constraint for Fourier modes in z------
  do z=2,nz/2
    do y=jjstart,jjend
      do x=iistart,iiend
	ux_k(x,y,nz+2-z)= ux_k(x,y,z)
	uy_k(x,y,nz+2-z)= uy_k(x,y,z)
	uz_k(x,y,nz+2-z)=-uz_k(x,y,z)
#ifdef SCALAR
	tha_k(x,y,nz+2-z)=-tha_k(x,y,z)
#endif
      enddo
    enddo 
  enddo
        
  do y=jjstart,jjend
    do x=iistart,iiend
      uz_k(x,y,1)=(0.,0.)
#ifdef SCALAR
      tha_k(x,y,1)=(0.,0.)
#endif
    enddo
  enddo 
	
end subroutine symm
