#ifdef SCALAR
subroutine v_ene(ux_k,uy_k,uz_k,tha_k)
#else
subroutine v_ene(ux_k,uy_k,uz_k)
#endif
!--------------------------------------------------------------
!       Calculation of the turbulent energies in the fully
!       periodic box 
!-------------------------------------------------------------- 

  use p3dfft_var
  use com

  implicit none
		
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: ux_k,uy_k,uz_k
#ifdef SCALAR
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: tha_k
#endif

  real(mytype) :: Ekin,Etha
  real(mytype) :: Ekin1,Etha1

  Ekin=0.
  Etha=0.
	
  Ekin1=0.
  Etha1=0.
	

!-------Factor 2 due to reality symmetry in kx	
  IF (taskid.ge.jproc) THEN

    do z=1,nz
      do y=jjstart,jjend 
        do x=iistart,iiend

          Ekin=Ekin+2*(ux_k(x,y,z)*conjg(ux_k(x,y,z))) &
	           +2*(uy_k(x,y,z)*conjg(uy_k(x,y,z))) &
	           +2*(uz_k(x,y,z)*conjg(uz_k(x,y,z)))
	             
#ifdef SCALAR
          Etha=Etha+2*(tha_k(x,y,z)*conjg(tha_k(x,y,z)))
#endif

        enddo
      enddo
    enddo

  ELSE

    do z=1,nz
      do y=jjstart,jjend
        do x=iistart,iistart

!---------kx=0 case
          Ekin=Ekin+ux_k(x,y,z)*conjg(ux_k(x,y,z)) &
                   +uy_k(x,y,z)*conjg(uy_k(x,y,z)) &
                   +uz_k(x,y,z)*conjg(uz_k(x,y,z))

#ifdef SCALAR
          Etha=Etha+tha_k(x,y,z)*conjg(tha_k(x,y,z))
#endif
        enddo

        do x=iistart+1,iiend

!---------kx ne 0 case again factor 2  
          Ekin=Ekin+2*ux_k(x,y,z)*conjg(ux_k(x,y,z)) &
 	           +2*uy_k(x,y,z)*conjg(uy_k(x,y,z)) &
	           +2*uz_k(x,y,z)*conjg(uz_k(x,y,z))
	             
#ifdef SCALAR
          Etha=Etha+2*tha_k(x,y,z)*conjg(tha_k(x,y,z))
#endif	
        enddo  
      enddo 
    enddo   

  ENDIF	

!-------Collecting energies from the slabs
  call MPI_REDUCE (Ekin,Ekin1,1,MPI_REAL,MPI_SUM,0,MPI_COMM_WORLD,ierr)
#ifdef SCALAR
  call MPI_REDUCE (Etha,Etha1,1,MPI_REAL,MPI_SUM,0,MPI_COMM_WORLD,ierr)	 
#endif
	
!-------Output---------------------------------	
  IF(taskid.eq.0)write(50,'(3(1x,e15.8))')time,Ekin1/2.,Etha1/2.
	
end subroutine v_ene
		
