#ifdef SCALAR
subroutine v_eps(ux_k,uy_k,uz_k,tha_k)
#else
subroutine v_eps(ux_k,uy_k,uz_k)
#endif

!--------------------------------------------------------------
!       Calculation of the energy dissipation rate in the fully
!       periodic box 
!-------------------------------------------------------------- 
  use p3dfft_var
  use com
	
  implicit none

  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: ux_k,uy_k,uz_k
#ifdef SCALAR
  complex(mytype), dimension(iistart:iiend,jjstart:jjend,1:nz) :: tha_k
#endif
	
  real(mytype) :: epsv,epst,epsv1,epst1
  real(mytype) :: kxsq,kysq,kzsq,ksq

  epsv=0.
  epst=0.
  epsv1=0.
  epst1=0.
	
	
!-------Factor 2 due to reality symmetry in kx	
  IF (taskid.ge.jproc) THEN

    do z=1,nz
      kzsq=kz(z)*kz(z)
        
      do y=jjstart,jjend 
        kysq=ky(y)*ky(y)

        do x=iistart,iiend
          kxsq=kx(x)*kx(x)
          ksq=kxsq+kysq+kzsq

          epsv=epsv+2*ksq*(ux_k(x,y,z)*conjg(ux_k(x,y,z))) &
                   +2*ksq*(uy_k(x,y,z)*conjg(uy_k(x,y,z))) &
                   +2*ksq*(uz_k(x,y,z)*conjg(uz_k(x,y,z)))
	             
#ifdef SCALAR
          epst=epst+2*ksq*(tha_k(x,y,z)*conjg(tha_k(x,y,z)))
#endif

        enddo
      enddo  
    enddo
   
  ELSE

    do z=1,nz
      kzsq=kz(z)*kz(z)
        
      do y=jjstart,jjend 
        kysq=ky(y)*ky(y)
        ksq=kysq+kzsq

        do x=iistart,iistart
          kxsq=kx(x)*kx(x)
          ksq=kxsq+kysq+kzsq

!---------kx=0 case 
          epsv=epsv+ksq*ux_k(x,y,z)*conjg(ux_k(x,y,z)) &
                   +ksq*uy_k(x,y,z)*conjg(uy_k(x,y,z)) &
                   +ksq*uz_k(x,y,z)*conjg(uz_k(x,y,z))
	             
#ifdef SCALAR
          epst=epst+ksq*tha_k(x,y,z)*conjg(tha_k(x,y,z))
#endif	

        enddo

        do x=iistart+1,iiend
          kxsq=kx(x)*kx(x)
          ksq=kxsq+kysq+kzsq

!---------kx ne 0 case again factor 2
          epsv=epsv+2*ksq*ux_k(x,y,z)*conjg(ux_k(x,y,z)) &
                   +2*ksq*uy_k(x,y,z)*conjg(uy_k(x,y,z)) &
                   +2*ksq*uz_k(x,y,z)*conjg(uz_k(x,y,z))

#ifdef SCALAR
          epst=epst+2*ksq*tha_k(x,y,z)*conjg(tha_k(x,y,z))
#endif
        enddo  
      enddo 
    enddo

  ENDIF
		  	
!-------Collecting energies from the slabs
  call MPI_REDUCE (epsv,epsv1,1,MPI_REAL,MPI_SUM,0,MPI_COMM_WORLD,ierr)
#ifdef SCALAR
  call MPI_REDUCE (epst,epst1,1,MPI_REAL,MPI_SUM,0,MPI_COMM_WORLD,ierr)
#endif

	
!-------Output---------------------------------	
!  IF(taskid.eq.0)write(51,'(2(1x,e15.8))')time,R_inv*epsv1
  IF(taskid.eq.0)write(51,'(3(1x,e15.8))')time,R_inv*epsv1,Pa_inv*epst1

end subroutine v_eps
		
