! This file is part of P3DFFT library
!
! Version 1.0 beta
!
! Copyright (C) 2006 Dmitry Pekurovsky, Giridhar Chukkapalli
!
!    P3DFFT is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.

!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

!----------------------------------------------------------------------------

c This file contains routines intended for initializing 1D FFT oprations
c as well as one-time initialization and clean-up

c Initialize backward complex-to-complex FFT 

      subroutine init_b_c(X,stride_x1,stride_x2,Y,stride_y1,
     &    stride_y2,N,m,flg_inplace)

      use fft_spec
      implicit none

      include 'p3dfft_incl_single.f'

      integer stride_x1,stride_x2,stride_y1,stride_y2,N,m,fflag
      complex(mytype) X(N*stride_x1+m*stride_x2),Y(N*stride_y1+m*stride_y2)
      logical flg_inplace


      call scft (1,X,stride_x1,stride_x2,Y,stride_y1,stride_y2,N,m,-1,1.0,
     1        caux1,cnaux,caux2,cnaux)  

      return
      end

!!--------------------------------------------------------------
c Initialize backward complex-to-real FFT 

      subroutine init_b_c2r(X,dimx,Y,dimy,N,m,flg_inplace)

      use fft_spec
      implicit none
      include 'p3dfft_incl_single.f'

      integer dimx,dimy,N,m,fflag
      complex(mytype) X((N/2+1)*m)
      real(mytype) Y(N*m)
      logical flg_inplace


      call scrft(1,X,dimx,Y,dimy,N,m,-1,1.0,
     &      raux1, rnaux1,raux2,rnaux2,raux3,1)  


      return
      end

!!--------------------------------------------------------------
c Initialize forward complex-to-complex FFT 

      subroutine init_f_c(X,stride_x1,stride_x2,Y,stride_y1,stride_y2,N,m,flg_inplace) 

      use fft_spec
      include 'p3dfft_incl_single.f'

      integer N,m,stride_x1,stride_x2,stride_y1,stride_y2,fflag
      complex(mytype) X(N*stride_x1+m*stride_x2),Y(N*stride_y1+m*stride_y2)
      logical flg_inplace


      call scft(1,X,stride_x1,stride_x2,Y,stride_y1,stride_y2,N,m,1,1.0, 
     1        caux1,cnaux,caux2,cnaux)


      return
      end

!!--------------------------------------------------------------
c Initialize forward real-to-complex FFT 

      subroutine init_f_r2c(X,dimx,Y,dimy,N,m,flg_inplace)

      use fft_spec
      include 'p3dfft_incl_single.f'

      integer dimx,dimy,N,m,fflag
      real(mytype) X(N*m)
      complex(mytype) Y((N/2+1)*m)
      logical flg_inplace


      call srcft(1,X,dimx,Y,dimy,N,m,1,1.0,
     &      raux1, rnaux1,raux2,rnaux2,raux3,1)           


      return
      end

!!--------------------------------------------------------------
c One-time initialization
C Initialize work arrays for 1

      subroutine init_work(nx,ny,nz)
      
      use fft_spec
      integer nx,ny,nz


      integer nyz
      nyz = max(ny,nz)
      if(nyz .le. 8192) then
         cnaux = 20000
      else 
         cnaux = 20000+1.14*nyz
      endif
      if(nyz .ge. 252) then
         cnaux = cnaux +(nyz+256)*64
      endif

      if(nx .le. 16384) then
         rnaux1 = 25000
         rnaux2 = 20000
      else
         rnaux1 = 20000+0.82*nx
         rnaux2=20000+0.57*nx
      endif

!      print *,'cnaux,rnaux1,rnaux2=',cnaux,rnaux1,rnaux2
      allocate(caux1(cnaux))
      allocate(caux2(cnaux))
      allocate(raux1(rnaux1))
      allocate(raux2(rnaux2))


      return
      end

!!--------------------------------------------------------------
c Clean-up routines for FFTW

      subroutine clean_x1

      use fft_spec
      
      return
      end

!!--------------------------------------------------------------

      subroutine clean_x2

      use fft_spec

      
      return
      end

!!--------------------------------------------------------------

      subroutine clean_x3

      
      return
      end

!!--------------------------------------------------------------
c Release work arrays for 1

      subroutine free_work
      
      use fft_spec

      deallocate(caux1)      
      deallocate(caux2)
      deallocate(raux1)
      deallocate(raux2)

      return
      end
