! This file is part of P3DFFT library
!
! Version 1.0 beta
!
! Copyright (C) 2006 Dmitry Pekurovsky, Giridhar Chukkapalli
!
!    P3DFFT is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.

!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

!----------------------------------------------------------------------------

c  3D FFT inverse transform with 2D domain decomposition
c
c  This version uses MPI_Alltoallv to exchange data between processors
c  In the second step, y planes are sent separately
c  The order of array elements in memory is the same in all stages: (x,y,z) 

c Input: XYZg - comlpex array, with y dimension contained entirely,
c               while x and y are block-distributed among processors in 2D grid
c Output: XgYZ - an array of real, x dimension is contained entirely within 
c processors memory  while z and y are block-distributed among processors 
c in 2D grid
c
c !!! CAUTION: In this version: all arrays may occupy the same memory space
c In this case their first elements should coincide. 
c Naturally, output overwrites input
c

      subroutine p3dfft_btran_c2r (XYZg,XgYZ,flg_inplace)

      use p3dfft_var
      use fft_spec
      implicit none

      real(mytype) XgYZ(nx_fft,jistart:jiend,kjstart:kjend)
c      complex(mytype) XZgY(iistart:iiend,ny,kjstart:kjend)
      complex(mytype) XYZg(iistart:iiend,jjstart:jjend,nz_fft)
      complex(mytype), allocatable :: buf(:,:,:)

      integer x,y,z,i,nx,ny,nz
      logical flg_inplace
c
      nx = nx_fft
      ny = ny_fft
      nz = nz_fft

      call init_work(nx,ny,nz)


      if(jproc .gt. 1) then

            call init_b_c(XYZg, iisize*jjsize, 1,
     &        XgYZ, iisize*jjsize, 1,nz,iisize*jjsize,flg_inplace)
c Transform in z dimension for all x and y
c

            call exec_b_c(XYZg, iisize*jjsize, 1,
     &        XgYZ, iisize*jjsize, 1,nz,iisize*jjsize)

c Exhange z-y pencils for all x
            call bcomm1(XgYZ,XgYZ)

      else
c In case of 1D decomposition do only FFT in Z without data exchange

            call init_b_c(XYZg,iisize*ny,1,XgYZ,
     &       iisize*ny, 1, nz,iisize*ny,flg_inplace)
            call exec_b_c(XYZg,iisize*ny,1,XgYZ,
     &       iisize*ny,1, nz,iisize*ny)
      endif

      call clean_x2

      allocate(buf(nxhp,jistart:jiend,kjstart:kjend))

      call init_b_c2r(buf,nxhp,XgYZ,nx,nx,jisize*kjsize,.false.)
      if(iproc .gt. 1) then
c
c Transform in y dimension for all x, one z-plane at a time
c
         call init_b_c(XgYZ,iisize,1,XgYZ,iisize,1,ny,iisize,.true.)
  
         do z=kjstart,kjend

               call btran_y_zplane(XgYZ,z-kjstart,iisize,kjsize,iisize,1,
     &            XgYZ,z-kjstart,iisize,kjsize,iisize,1,ny,iisize) 
c             
c              call sexec_b_c(XZgY(iistart,1,z),iisize,1,
c     &              XZgY(iistart,1,z),iisize,1,ny,iisize)

         enddo

         call bcomm2(XgYZ,buf)

      else

            call init_b_c(XgYZ,nxh,1,buf,nxhp,1,ny,nxh,.false.)

c In case of 1D decomposition do only FFT in Y without data exchange
            do z=kjstart,kjend

               call btran_y_zplane(XgYZ,z-kjstart,nxh,kjsize,nxh,1,buf,
     &      z-kjstart,nxhp,kjsize, nxhp,1,ny,nxh)
c               call sexec_b_c(XgYZ(1,1,z),nxh,1,buf(1,1,z),nxhp,1,ny,nxh)

               do y=1,ny
                  buf(nxhp,y,z)=cmplx(imag(buf(1,y,z)),0.0)
                  buf(1,y,z) = cmplx(real(buf(1,y,z)),0.0)
               enddo
            enddo

      endif

      call clean_x2

c Perform Complex-to-real FFT in x dimension for all y and z

      call exec_b_c2r(buf,nxhp,XgYZ,nx,nx,jisize*kjsize)

      call clean_x1

      deallocate(buf)

      call free_work

      return
      end
