! This file is part of P3DFFT library
!
! Version 1.0 beta
!
! Copyright (C) 2006 Dmitry Pekurovsky, Giridhar Chukkapalli
!
!    P3DFFT is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.

!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

!----------------------------------------------------------------------------

c Communication Module
c 
c Contains 4 routines for forward and backward exchanges inrows and columns 
c of processors. Uses MPI_Alltoallv routine
c

c Transpose X and Y pencils

      subroutine fcomm1(source,dest)

      use p3dfft_var
      implicit none

      complex(mytype) source(nxhp,jistart:jiend),dest(iistart:iiend,ny_fft)

      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) sendbuf(jisize*nxh)
      integer x,y,i,ierr
      integer*8 position

      tp1 = tp1 - MPI_Wtime() 
   
c Pack the real part of nx/2 frequency into the imaginary part of 0 frequency
c (both have imaginary part zero due to conjugte symmetry of a real-to-complex 
c transform)

      do y=jistart,jiend
         source(1,y) = cmplx(real(source(1,y)),real(source(nxhp,y)))
      enddo

c Pack the send buffer for exchanging y and x (within a given z plane ) into sendbuf
      position = 1

      do i=0,iproc-1
         do y=jistart,jiend
            do x=iist(i),iien(i)
               sendbuf(position) = source(x,y)
               position = position +1
            enddo
         enddo
      enddo
      tp1 = tp1 + MPI_Wtime() 
      
      t1 = t1 - MPI_Wtime() 
         
c Exchange the y-x buffers

      call mpi_alltoallv(sendbuf,IfSndCnts, IfSndStrt,mpi_byte, 
     &     dest,IfRcvCnts, IfRcvStrt,mpi_byte,mpi_comm_row,ierr)
      t1 = MPI_Wtime() + t1

      return
      end

c Transpose Y and Z pencils

      subroutine fcomm2(source,dest)

      use p3dfft_var
      implicit none

      complex(mytype) source(iistart:iiend,ny_fft,kjstart:kjend)
      complex(mytype) dest(iistart:iiend,jjstart:jjend,nz_fft)
      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) sendbuf(iisize*kjsize*ny_fft)
      integer x,z,y,i,ierr
      integer*8 position

c Pack send buffers for exchanging y and z for all x at once 
      position = 1
      t2 = t2 - MPI_Wtime()
 
      do i=0,jproc-1
         do z=kjstart,kjend
            do y=jjst(i),jjen(i)
               do x=iistart,iiend
                  sendbuf(position) = source(x,y,z)
                  position = position+1
               enddo
            enddo
         enddo
      enddo
         
c Exchange y-z buffers

      call mpi_alltoallv(sendbuf,KfSndCnts, KfSndStrt,mpi_byte,
     &     dest,KfRcvCnts, KfRcvStrt,mpi_byte,mpi_comm_col,ierr)
      t2 = MPI_Wtime() + t2
         
      return
      end


c Transpose back Z to Y pencils

      subroutine bcomm1 (source,dest)
      use p3dfft_var
      implicit none

      complex(mytype) source(iistart:iiend,jjstart:jjend,nz_fft)
      complex(mytype) dest(iistart:iiend,ny_fft,kjstart:kjend)

      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) recvbuf1(iisize*kjsize*ny_fft)
      integer x,y,z,i,ierr
      integer*8 position

      t3 = t3 - MPI_Wtime() 
      call mpi_alltoallv(source,JrSndCnts, JrSndStrt,mpi_byte,
     &     recvbuf1,JrRcvCnts, JrRcvStrt,mpi_byte,mpi_comm_col,ierr)
      
c Unpack receive buffers into XZgY

      position=1
      do i=0,jproc-1
         do z=kjstart,kjend
            do y=jjst(i),jjen(i)
               do x=iistart,iiend
                  dest(x,y,z) = recvbuf1(position)
                  position = position+1
               enddo
            enddo
         enddo
      enddo
      
      t3 = t3 + MPI_Wtime() 
      
      return
      end


c Transpose back Y to X pencils

      subroutine bcomm2(source,dest)

      use p3dfft_var
      implicit none

      complex(mytype) dest(nxhp,jistart:jiend,kjstart:kjend)
      complex(mytype) source(iistart:iiend,ny_fft,kjstart:kjend)
      real(8) t1,t2,t3,t4,tp1
      common /timers/ t1,t2,t3,t4,tp1
      complex(mytype) recvbuf(jisize*nxh)
      integer x,y,z,i,ierr
      integer*8 position

c Exchange x-z buffers
      
      do z=kjend,kjstart,-1
         t4 = t4 - MPI_Wtime() 
         call mpi_alltoallv (source(iistart,1,z),KrSndCnts, KrSndStrt,
     &      mpi_byte, recvbuf,KrRcvCnts,KrRcvStrt,mpi_byte,   
     &      mpi_comm_row,ierr)
         t4 = t4 + MPI_Wtime() 

c Unpack receive buffers into XgYZ

         position=1
         do i=0,iproc-1
            do y=jistart,jiend
               do x=iist(i),iien(i)
                  dest(x,y,z) = recvbuf(position)
                  position = position+1
               enddo
            enddo
         enddo

c Move the real part of nx/2 frequency to its proper location to prepare
c for the inverse X transform

         do y=jistart,jiend
            dest(nxhp,y,z)=cmplx(imag(dest(1,y,z)),0.0)
            dest(1,y,z) = cmplx(real(dest(1,y,z)),0.0)
         enddo
         
      enddo
      
      return
      end


