! This file is part of P3DFFT library
!
! Version 1.0 beta
!
! Copyright (C) 2006 Dmitry Pekurovsky, Giridhar Chukkapalli
!
!    P3DFFT is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.

!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

!----------------------------------------------------------------------------

c 3D FFT Forward transform, 2D domain decomposition
c
c  This version uses MPI_Alltoallv to exchange data between processors
c  In the first step, y planes are sent separately
c  The order of array elements in memory is the same in all stages: (x,y,z) 
c
c Input: XgYZ - an array of real, x dimension is contained entirely within 
c  processors memory  while z and y are block-distributed among processors 
c  in 2D grid XgYZ 
c Output: XYZg - comlpex array, with y dimension contained entirely,
c               while x and y are block-distributed among processors in 2D grid
c
c !!! CAUTION: In this version: all arrays may occupy the same memory space
c In this case their first elements should coincide. 
c Naturally, output overwrites input

      subroutine p3dfft_ftran_r2c (XgYZ,XYZg,flg_inplace)

      use p3dfft_var
      use fft_spec
      implicit none

      real(mytype) XgYZ(nx_fft,jistart:jiend,kjstart:kjend)
c      complex(mytype) XYgZ(iistart:iiend,ny,kjstart:kjend)
      complex(mytype) XYZg(iistart:iiend,jjstart:jjend,nz_fft)
      complex(mytype), allocatable :: buf(:,:)
      integer x,y,z,i,err,nx,ny,nz
      logical flg_inplace

      nx = nx_fft
      ny = ny_fft
      nz = nz_fft

c
c FFT transform in x for all z and y
c

      allocate(buf(nxhp,jisize),stat=err)
      if(err .ne. 0) then
         print *,'Error ',err,' allocating array buf'
      endif

      call init_work(nx,ny,nz)

      call init_f_r2c(XgYZ,nx,buf,nxhp,nx,jisize,.true.) 

      if(iproc .eq. 1) then
         call init_f_c(buf,nxhp,1,buf,nxh,1,ny,nxh,.false.)
      endif

      do z=kjstart,kjend

         call exec_f_r2c(XgYZ(1,jistart,z),nx,buf,nxhp,nx,jisize)

         if(iproc .gt. 1) then
            call fcomm1(buf,XgYZ(1,jistart,z))
         else
c In case of 1D decomposition do only FFT in Y without data exchange
            do y=1,ny
               buf(1,y) = cmplx(real(buf(1,y)),real(buf(nxhp,y)))
            enddo

            call exec_f_c(buf,nxhp,1,XgYZ(1,1,z),nxh,1,ny,nxh)

         endif
      enddo

      call clean_x1
      
      if(iproc .gt. 1) then

c Perform FFT in y for all x for a given z plane
         call init_f_c(XgYZ,iisize,1,XYZg,iisize,1,ny,iisize,flg_inplace)

         do z=kjstart,kjend

            call ftran_y_zplane(XgYZ,z-kjstart,iisize,kjsize,iisize,1,
     &            XYZg,z-kjstart,iisize,kjsize,iisize,1,ny,iisize)
c            call exec_f_c(XgYZ(1,jistart,z),iisize,1,
c     &     XgYZ(1,jistart,z),iisize,1,ny,iisize)

         enddo
      endif

      deallocate(buf)
      call clean_x2

      if(jproc .gt. 1) then

         call fcomm2(XYZg,XYZg)

c Transform in z for all x and y

         call init_f_c(XYZg,iisize*jjsize, 1,
     &   XYZg,iisize*jjsize, 1,nz,iisize*jjsize,.true.)

         call exec_f_c(XYZg,iisize*jjsize, 1,
     &   XYZg,iisize*jjsize, 1,nz,iisize*jjsize)

      else
c In case of 1D decomposition do only FFT in Y without data exchange
         call init_f_c(XYZg,iisize*ny,1,XYZg,
     &    iisize*ny,1, nz,iisize*ny,.true.)
         call exec_f_c(XYZg,iisize*ny,1,XYZg,
     &    iisize*ny,1, nz,iisize*ny)
      endif

      call clean_x2
      call free_work

      return
      end

